require 'json'
require 'sinatra'
require 'sinatra/reloader'
require 'sass'
require 'coffee-script'
require 'image_size'

require './hh.rb'
require './image_tools.rb'

# Pull info out of HH::{Photo,Gallery} objects into hashes for serialization.
helpers do
  def hash_img(img)
    id     = img.path.sub /^public./, '' # XXX My word this is gross
    thumb  = img.thumb.path.sub /^public./, ''
    w, h   = img.size
    tw, th = img.thumb.size
    {
      :id    => id,
      :thumb => img.has_thumb? ? thumb : nil,
      :w     => w,
      :h     => h,
      :tw    => tw,
      :th    => th,
      :rot   => ImageTools.rotate_to(img.path),
      :gal   => false
    }
  end
  def hash_dir(dir)
    tw, th = dir.montage.size
    {
      :id      => dir.path.sub(/^public./, ''), # XXX More utter grossness.
      :montage => dir.montage.exists? ? dir.montage.path.sub(/^public./, '') : nil,
      :tw      => tw,
      :th      => th,
      :gal     => true
    }
  end
end

get '/' do
  File.read('index.html')
end

get '/css/gal.css' do
  sass :hh
end

get '/js/script.js' do
  coffee :hh
end

get '/gals/?*' do
  content_type :json

  dir = "public/" + ( !params[:splat][0].empty? ? File.join(params[:splat]) : 'photos' )
  HaruHaru::Gallery.new(dir).listing.collect{|f|
    f.is_a?(HaruHaru::Photo) ? hash_img(f) : hash_dir(f)
  }.to_json
end

get '/make/thumb/*' do
  content_type :json

  img = HaruHaru::Photo.new('public/' + File.join(params[:splat]))
  img.make_thumb
  hash_img(img).to_json
end

get '/make/montage/*' do
  content_type :json

  gal = HaruHaru::Gallery.new("public/" + File.join(params[:splat]))
  gal.make_montage
  hash_dir(gal).to_json
end
