require 'image_size'
require './image_tools.rb'

class HaruHaru
  class Image
    attr_reader :path
    def initialize(path)
      @path = path
    end

    def size
      # XXX Cache results somewhere?
      if not File.file? @path
        [nil, nil]
      elsif File.size(@path) <= 819200 # 800k, seems about right.
        begin
          # Blech, reading a whole image into memory is ridiculous.
          ImageSize.new(File.open(@path, 'rb').read).get_size
        rescue Exception
          [nil, nil]
        end
      else
        size = `identify -format '%[fx:w]x%[fx:h]' "#{@path}" 2> /dev/null`.chomp
        size.length ? size.split(/x/).map{|d| d.to_i} : [nil, nil]
      end
    end
  end

  class Thumb < Image
    def exists?
      File.file? self.path
    end
  end
  class Photo < Image
    def thumb
      Thumb.new File.join(File.dirname(@path), '.thumbs', File.basename(@path))
    end
    def has_thumb?
      self.thumb.exists?
    end
    def make_thumb
      return self.thumb.path if File.file?(self.thumb.path)
      w, h = self.size
      tw = w > 250 ? ( w < h ? 250 / h.to_f * w : 250 ) : w
      th = h > 250 ? ( h < w ? 250 / w.to_f * h : 250 ) : h
#      $stderr.puts "Creating thumb for #{@path} from #{w}x#{h} for #{tw.to_i}x#{th.to_i}"
      ImageTools.thumbnail @path, self.thumb.path, tw.to_i, th.to_i
      self.thumb
    end
  end
  class Gallery
    attr_reader :path
    def initialize(path)
      @path = path
    end
    def listing
      # Get JPGs or dirs, sort 'em, make paths then divide into dirs & files
      fls = Dir.open(@path).find_all{|f|
        path = File.join(@path, f)
        # Get images or dirs - TODO filter out dirs without images
        (File.file?(path) and f =~ /\.(jpe?g|gif)$/i) or (f !~ /^\./ and File.directory?(path))
      }.sort.collect{|f| File.join(@path, f)}.partition{|f| File.directory?(f)}.flatten.collect{|f|
        File.file?(f) ? Photo.new(f) : Gallery.new(f)
      }
    end
    # XXX Image subclass?
    def montage
      Photo.new(File.join @path, 'montage.jpg').thumb
    end
    def has_montage?
      self.montage.exists?
    end
    def make_montage
      montage = self.montage

      return montage.path if montage.exists?

      fls = Dir.open(@path).collect{|f| File.join(@path, f)}.find_all{|f|
        File.file?(f) and f =~ /\.jpe?g$/i
      }

      return if fls.length == 0 # No images :S

      # Base montages on thumbnails so we don't eat all the memory.
      for_montage = fls.sample(4).collect{|i| Photo.new(i) }
      for_montage.each{|i| i.make_thumb}

      ImageTools.quick_montage(for_montage.collect{|i| i.thumb.path}, montage.path)

      return montage
    end
  end

  def initialize(dir)
    @dir = dir
  end

  def photo(path)
    Photo.new( self.get_path path )
  end

  def gallery
    Gallery.new( @dir )
  end

  protected

  def get_path(path)
    path = File.join(@dir, path)
    raise IOError, "No such - #{path}" unless File.file?(path)
    path
  end
end
