require 'RMagick'
require 'fileutils'
require 'exifr'

module ImageTools
  class << self
    include Magick

    def thumbnail(source, target, width, height = nil)
      return nil unless File.file?(source)
      height ||= width

      FileUtils.mkdir_p(File.dirname(target))

      system *[%w(convert -thumbnail), "#{width}x#{height}", source, target].flatten
    end

    def quick_montage(sources, target)
      return nil unless sources.all?{|f| File.file?(f)} # XXX May want to die louder?

      FileUtils.mkdir_p(File.dirname(target))
      system *[%w(montage -geometry 125x125 -tile 2x2), sources, target].flatten
    end
    def rotate_to(img)
      begin
        type = EXIFR::JPEG.new(img).orientation
      rescue
        return 0
      end
      # via http://exifr.rubyforge.org/api/classes/EXIFR/TIFF/Orientation.html#M000011
      case type
#        when EXIFR::TIFF::TopRightOrientation    ; img.flop
        when EXIFR::TIFF::BottomRightOrientation ; 180
#        when EXIFR::TIFF::BottomLeftOrientation  ; img.flip
#        when EXIFR::TIFF::LeftTopOrientation     ; img.rotate(90).flop
        when EXIFR::TIFF::RightTopOrientation    ; 90
#        when EXIFR::TIFF::RightBottomOrientation ; img.rotate(270).flop
        when EXIFR::TIFF::LeftBottomOrientation  ; 270
        else
          0
      end
    end
  end
end

