
// A FIFO stack object which ensures that no more than this.max requests are
// being made at any one time. This allows hundreds of requests to be made
// with no more than this.max in flight so it's nice on servers and bandwidth.

function RequestStack(options) {
    options       = options         || {};
    this.max      = options.max     || 5;
    this.requests = options.request || [];
    this.stack    = options.stack   || [];
}
RequestStack.COMPLETE = 4;
RequestStack.prototype = {
    make_requests: function() {
        try {

            // Add pending requests to the request stack.
            while( this.stack.length < this.max && this.requests.length != 0 ) {
                var req = this.requests.shift();
                this.stack.push( req );
                req.send(null);
            }

            var self = this;
            // Find completed requests.
            var reqsdone = this.stack.filter(function(req) {
                return req.readyState == RequestStack.COMPLETE;
            }).map(function(req) { return self.stack.indexOf(req); });
            
            // Remove completed requests from request stack.
            for(var idx = 0; idx < reqsdone.length; idx++)
                this.stack.splice( reqsdone[idx], 1 );
            
            if(this.stack.length == 0 && this.requests.length == 0)
                this.end();

        } catch(e) {
            console.error("[make_requests] bugness - ", e, '[I am ', this ,']');
            this.end();
        }
    },
    add: function(req) {
        var self = this;
        // If no worker has started or the last has finished but not been cleared.
        if(!this.worker)
            this.worker = setInterval(function() { self.make_requests(); }, 250);
        
        this.requests.push(req);
    },
    end: function() {
        clearInterval(this.worker);
        this.worker = null;
    }
};
