$ ->
    Thumb = Backbone.Model.extend {}

    Listing = Backbone.Collection.extend
        model: Thumb,
        url: ->
            window.location.pathname + 'gals/' + @.gal
        initialize: (models, options) ->
            @gal = options.gal || ''

    # Pre-populate with a root listing?
    list = new Listing

    Gallery = Backbone.View.extend
        initialize: ->
            _.bindAll @, 'showPic', 'handleKeys'
        render: ->
            viewer = $('#viewer')
            viewer.css(
                width:  document.documentElement.clientWidth
                height: document.documentElement.clientHeight
            ).click -> viewer.hide()
            viewer.find('.gallery').empty()

            list.each (thumb, idx) ->
                thumb.set index: "img-#{idx}"
                imageDom = $('#image-tmpl').tmpl {id: thumb.get 'index'}
                imageDom.find('img').load -> $(@).fadeIn()
                        .parent().css height: $('#viewer').attr('clientHeight')
                viewer.find('.gallery').append imageDom

        scaleImage: (width, height) ->
            # XXX Doesn't work cross browser, should probably use jQuery.stuff
            [ww, wh] = (document.documentElement["client#{d}"] for d in ['Width', 'Height'])
            [iw, ih] = [width, height]

            image = {}

            # Scale one dimension and have the browser do the rest as necessary.
            image.width  = ww if iw > ww
            # Manually scale height if it's more than the window width
            image.height = Math.floor ih / iw * ww if ih > wh and iw > ww
            image.height = wh if ih > wh and !image.height
            # Scale on height if the scaled height is still too big.
            if image.width == ww and image.height >= wh
                image.height = wh
                delete image.width

            return image

        showPic: (thumb) ->
            image     = @scaleImage thumb.data('w'), thumb.data('h')
            image.src = thumb.data('img')

            @enableNav thumb.attr('id')

            $('#viewer').show()

            imageIdx = @img.get('index')
            $("##{imageIdx}").attr image
            # XXX jQuery infinite recursion bug? $("i#{thumb.get('index')}").attr

            return @slideTo imageIdx

        slideTo: (imageIdx) ->
            # jQuery('#viewer .gallery').animate({top: '-=423'}, 500)
            moveTo = $("##{imageIdx}").parent().attr('offsetTop')
            $('#viewer .gallery').animate {top: if moveTo <= 0 then 0 else -moveTo}, 750, =>
                # Get rid of other images to keep memory down.
                $(".gallery img[id!=#{imageIdx}]").attr src: ''
                @rotatePic 'l' if @img.get('rot') is 90

        enableNav: (curImg) ->
            # Would be nicer with .one but breaks down at boundaries
            $(document).keydown @handleKeys unless @img
            @img = list.get(curImg)

        handleKeys: (e) ->
            if e.which is 27 # i.e Esc
                return @disable()
            key = String.fromCharCode(e.which).toLowerCase()
            if key is 'j' or key is 'k'
                @nextPic(key)
            else if key is 'h' or key is 'l'
                @rotatePic(key)
            else if key is 's'
                @slideShow()

        nextPic: (key) ->
            m = list.at list.indexOf(@img) + ( if key is 'j' then 1 else -1 )
            if m and !m.get 'gal'
                @showPic $(document.getElementById m.id)
            else
                return false

        rotatePic: (key) ->
            if @img.get 'rotated'
                return
            if key is 'l'
                curImg = $ '#' + @img.get('index')
                [h, w] = [curImg.height(), curImg.width()]
                newDim = @scaleImage h, w
                newImg = if newDim.height then {width: newDim.height, height: h / w * h} else {height: newDim.width, width: w / h * w}
                curImg.animate newImg, 300, =>
                    @img.set 'rotated': yes
                    curImg.css # addClass .rotateRight = no workee
                        '-webkit-transition': '-webkit-transform .3s ease-in-out'
                        '-webkit-transform':  'rotate(90deg)'

        slideShow: ->
            @sliding = if @sliding is on then off else on
            if @sliding
                @slideWorker = setInterval (=> @nextPic 'j'), 3000
            else
                clearTimeout @slideWorker

        disable: ->
            $(document).unbind 'keydown', @handleKeys
            @img = null
            clearTimeout @slideWorker
            $('#viewer').hide()

    gal = new Gallery

    HaruHaru = Backbone.View.extend
        events:
            'click .gal': 'loadThumbs'
            'click .pic': 'showPic'

        initialize: ->
            _.bindAll @, 'render', 'getThumbs'
            @reqstack = new RequestStack

        render: ->
            @loadImages()
            $('#main h1').text list.gal || 'The Gallery'

        loadImages: ->
            $(@el).empty().append $('#thumb-tmpl').tmpl list.map @getThumbs

        makeImage: (needed) ->
            @reqstack.add new MakeImageReq needed

        getThumbs: (img) ->
            caption = img.id.match(/\/([^\/]+)(?:[.]\w+)?$/)[1]
            # 10 = Num of placeholders, could this be a collection?
            placeHolder = "/img/placeholders/lorempixum-#{Math.floor Math.random() * 10}.jpg"

            if img.get 'gal'
                @makeImage(i: 'montage', s: img.id) unless img.get('montage')
                {
                    id:  img.id
                    src: img.get('montage') || placeHolder
                    cap: caption
                    gal: true
                }
            else
                @makeImage(i: 'thumb', s: img.id) unless img.get('thumb') or img.id.match /[.]gif$/i
                {
                    id    : img.id
                    src   : img.get('thumb') || placeHolder
                    w     : img.get('tw') || 250
                    h     : img.get('th') || 250
                    realw : img.get('w') || ''
                    realh : img.get('h') || ''
                    cap   : caption
                    gal   : false
                }

        loadGal: (newGal) ->
            list   = new Listing {}, { gal: newGal }
            list.fetch
                success: ->
                    album.render()
                    gal = new Gallery
                    gal.render()

        loadThumbs: (evt) ->
            newGal = evt.target.id
            window.location.hash = "gal/#{newGal}"
            $('#main h1').text 'Loading gallery ...'

        showPic: (evt) ->
            gal.showPic( $(evt.target) )

    album = new HaruHaru {el: $ '#thumbs'}
    # Let the NavSupport trigger rendering.

    class MakeImageReq
        constructor: (@needed) ->
            _.bindAll @, 'makeImage'
            # Useful for debugging weirdness.
            throw "Nothing needed!" unless @needed.i and @needed.s

        send: ->
            $.getJSON("/make/#{@needed.i}/#{@needed.s}", @makeImage)
             .complete => @readyState = RequestStack.COMPLETE

        makeImage: (json) ->
            return unless imgSrc = json.thumb || json.montage
            # console.log "Made image for: #{json.id}"
            # Don't use a selector as the id will confuse Sizzle.
            $(document.getElementById json.id).data(json).attr(
                src:    imgSrc
                width:  json.tw
                height: json.th
            ).parent().css(width: "#{json.tw}px")

    NavSupport = Backbone.Controller.extend
        initialize: ->
            # Handle going all the way back.
            @route /^$/, 'top', -> album.loadGal ""

        routes:
            "gal/*path": "updateGal"

        updateGal: (newGal) ->
            gal.disable()
            album.loadGal newGal

    new NavSupport
    Backbone.history.start()

